//
//  M5FrostedWindow.m
//  M5FrostedWindow
//
//  Created by Mathew Huusko V on 7/10/13.
//  Copyright (c) 2013 mhuusko5. All rights reserved.
//

#import "M5FrostedWindow.h"

@implementation M5FrostedWindow

- (id)initWithContentRect:(NSRect)contentRect styleMask:(NSUInteger)aStyle backing:(NSBackingStoreType)bufferingType defer:(BOOL)flag {
	self = [super initWithContentRect:contentRect styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
    
	[self setLevel:NSFloatingWindowLevel];
	[self setCollectionBehavior:NSWindowCollectionBehaviorCanJoinAllSpaces];
	[self setMovableByWindowBackground:NO];
	[self setOpaque:NO];
    
	[self setStyleMask:NSTitledWindowMask];
    
	[self setHasShadow:NO];
	[self setBackgroundColor:[NSColor clearColor]];
    
	[self performSelector:@selector(setupBackgroundUpdating) withObject:nil afterDelay:0.1];
    
	return self;
}

static float updateInterval = 0.1f; //2.0f;
static float blurRadius = 3.0f;

- (void)setupBackgroundUpdating {
	NSRect backgroundImageFrame = [self.contentView frame];
	backgroundImageFrame.size.width *= blurRadius;
	backgroundImageFrame.size.height *= blurRadius;
	backgroundImageFrame.origin.x += ([self.contentView frame].size.width - backgroundImageFrame.size.width) / 2;
	backgroundImageFrame.origin.y += ([self.contentView frame].size.height - backgroundImageFrame.size.height) / 2;
    
	backgroundImage1 = [[NSImageView alloc] initWithFrame:[self.contentView frame]];
	[backgroundImage1 setImageScaling:NSImageScaleAxesIndependently];
	[backgroundImage1 setImageFrameStyle:NSImageFrameNone];
	[backgroundImage1 setImage:[self blurImage:[self takeBackgroundImage]]];
    
	backgroundImage2 = [[NSImageView alloc] initWithFrame:[self.contentView frame]];
	[backgroundImage2 setImageScaling:NSImageScaleAxesIndependently];
	[backgroundImage2 setImageFrameStyle:NSImageFrameNone];
	[backgroundImage2 setAlphaValue:0.0];
    
	[self.contentView addSubview:backgroundImage1];
	[self.contentView addSubview:backgroundImage2];
    
	//TODO: This timer action doesn't get called when the window is dragged - there must be a workaround here...
	[NSTimer scheduledTimerWithTimeInterval:updateInterval target:self selector:@selector(updateBackgroundImage) userInfo:nil repeats:YES];
}

- (void)updateBackgroundImage {
	[backgroundImage2 setImage:[self blurImage:[self takeBackgroundImage]]];
    
	//TODO: With a longer updateInterval (which is really necessary for performance reasons, have these two image view fade between each other)
	[backgroundImage2 setAlphaValue:1.0];
	[backgroundImage1 setAlphaValue:0.0];
    
	NSImageView *tempBackgroundImage = backgroundImage2;
	backgroundImage2 = backgroundImage1;
	backgroundImage1 = tempBackgroundImage;
}

//TODO: Would like to tint color as well as blur it
- (NSImage *)blurImage:(NSImage *)image {
	CIImage *inputImage = [CIImage imageWithData:[image TIFFRepresentation]];
    
	CIFilter *filter = [CIFilter filterWithName:@"CIGaussianBlur"];
    
	[filter setValue:inputImage forKey:kCIInputImageKey];
	[filter setValue:[NSNumber numberWithFloat:blurRadius] forKey:@"inputRadius"];
    
	CIImage *outputImage = [filter valueForKey:kCIOutputImageKey];
    
	NSImage *blurredImage = [[NSImage alloc] initWithSize:NSRectFromCGRect([outputImage extent]).size];
    
	[blurredImage lockFocus];
    
	[outputImage drawAtPoint:NSZeroPoint fromRect:NSRectFromCGRect([inputImage extent]) operation:NSCompositeCopy fraction:1.0];
    
	[blurredImage unlockFocus];
    
	return blurredImage;
}

- (NSImage *)takeBackgroundImage {
	CGWindowID windowID = (CGWindowID)[self windowNumber];
	NSRect contentRectInScreen = [self convertRectToScreen:[self.contentView frame]];
	CGRect windowRect = NSRectToCGRect(contentRectInScreen);
	windowRect.origin.y = NSMaxY([[self screen] frame]) - NSMaxY(contentRectInScreen);
	CGImageRef captureImage = CGWindowListCreateImage(windowRect, kCGWindowListOptionOnScreenBelowWindow, windowID, kCGWindowImageDefault);
    
	NSBitmapImageRep *bitmapRep = [[NSBitmapImageRep alloc] initWithCGImage:captureImage];
	NSImage *image = [[NSImage alloc] init];
	[image addRepresentation:bitmapRep];
	CGImageRelease(captureImage);
    
	return image;
}

- (BOOL)canBecomeKeyWindow {
	return YES;
}

- (BOOL)isMainWindow {
	return YES;
}

@end
